﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

// For Hex format testing
using System.Globalization;

using AIOUSBNet;  // the namespace exposes the AIOUSB Class interface 

// You must also add a reference to the AIOUSBnet.dll in the project settings


namespace Sample1
{
    public partial class Form1 : Form
    {
        // One and only Device Index:
        public UInt32 DeviceIndex;
        public UInt32 numPins = 16;
        public UInt32 numPorts = 2;

        // Interval Timer fou GUI update:
        // (Note: A Windows Forms timer designed for single threaded environments not a System Timer 55 ms min res)
        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();

        // Array of controls for GUI updates:
        public CheckBox[] DIOCheckBox = new CheckBox[16];
        public TextBox[] DIOText = new TextBox[2];
        
        public TrackBar[] DACTrack = new TrackBar[2];
        public TextBox[] DACText = new TextBox[2];

        // For bitmasks:
        byte[] OutMask = new byte[] { 0xFF, 0xFF };  // for DAC DIO sample default to output at startup

       // Data:
        byte[] DIOData = new byte[] { 0, 0 };
        ushort[] DACData = new ushort[] { 0, 0 };


        // State flags:
        byte bTristateAll = 0;  // 0 = false all bits not tristated
        public bool[] bModeOut = new bool[2];
        
        public Form1()
        {
            InitializeComponent();

            // Array of the LED style checkboxes:
            DIOCheckBox[0] = checkBoxA0_1;
            DIOCheckBox[1] = checkBoxA0_2;
            DIOCheckBox[2] = checkBoxA0_3;
            DIOCheckBox[3] = checkBoxA0_4;
            DIOCheckBox[4] = checkBoxA0_5;
            DIOCheckBox[5] = checkBoxA0_6;
            DIOCheckBox[6] = checkBoxA0_7;
            DIOCheckBox[7] = checkBoxA0_8;
            DIOCheckBox[8] = checkBoxB0_9;
            DIOCheckBox[9] = checkBoxB0_10;
            DIOCheckBox[10] = checkBoxB0_11;
            DIOCheckBox[11] = checkBoxB0_12;
            DIOCheckBox[12] = checkBoxB0_13;
            DIOCheckBox[13] = checkBoxB0_14;
            DIOCheckBox[14] = checkBoxB0_15;
            DIOCheckBox[15] = checkBoxB0_16;

            DIOText[0] = textDIOByte0;
            DIOText[1] = textDIOByte1; 
            
            DACTrack[0] = trackBarDAC0;
            DACTrack[1] = trackBarDAC1;

            DACText[0] = textDAC0;
            DACText[1] = textDAC1;
        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            // Called before Form is displayed Initialize resources:

            // Initialize default Device Only:
            DeviceIndex = AIOUSB.diOnly;

            // Device data:
            UInt32 Status;
            UInt32 PID = 0;
            UInt32 NameSize = 256;
            string strName = "name";
            UInt32 DIOBytes = 0;
            UInt32 Counters = 0;


            UInt32 ERROR_SUCCESS = 0;
            bool deviceIndexValid = false;

            // Get The Device Information test for valid device found:
            Status = AIOUSB.QueryDeviceInfo(DeviceIndex, out PID, ref NameSize, out strName, out DIOBytes, out Counters);
            if (Status == ERROR_SUCCESS && PID >= 0x8140 && PID <= 0x815F) // AIO not AI
            {
                deviceIndexValid = true;
            }
            else
            {
                // If Only device is not valid then Launch connect device dialog box:
                // New parent aware subform:
                FormDetect DetectSubForm = new FormDetect(this);
                DetectSubForm.ShowDialog();

                Status = AIOUSB.QueryDeviceInfo(DeviceIndex, out PID, ref NameSize, out strName, out DIOBytes, out Counters);
                if (Status == ERROR_SUCCESS && PID >= 0x8140 && PID <= 0x815F)
                    deviceIndexValid = true;
            }

            if (!deviceIndexValid)
            {
                // No valid device found could just exit
                // this.Close();
            }

            // Init values:
            for (int i = 0; i < numPorts; i++)
            {
                DIOData[i] = 0xFF;
                bModeOut[i] = true;  // Default power up Output Mode
            }

            Status = AIOUSB.DIO_ReadAll(DeviceIndex, DIOData); 
            Status = AIOUSB.DIO_Configure(DeviceIndex, bTristateAll, OutMask, DIOData);

            // Display current DIO settings:
            DIOText[0].Text = DIOData[0].ToString("X2");
            DIOText[1].Text = DIOData[1].ToString("X2");

            textBoxStatus.Text = "This sample just writes DAC and DIO data in one function call.\r\n Enter settings with the GUI controls or enter a valid Hex number.\r\n Then press the 'Write All' button.";
        }
         
        private void btnDevice_Click(object sender, EventArgs e)
        {
            // Launch connect device dialog box
            // Switch between multiple devices or reconnect

            // New parent aware subform:
            FormDetect DetectSubForm = new FormDetect(this);
            DetectSubForm.ShowDialog();
        }

        private void btnWriteAll_Click(object sender, EventArgs e)
        {
            // Validate Hex inputs or prompt for correction(s):
            // If valid then write all the DAC and DIO data in one call and report status: 

            uint Status = 0; 

            bool bValid = true;
            Byte tempByte0, tempByte1;
            UInt16 tempShort0, tempShort1;
            NumberStyles styles = System.Globalization.NumberStyles.HexNumber;  // composite number style
            IFormatProvider format = System.Globalization.NumberFormatInfo.CurrentInfo; // can use null as well

            // Test for valid hex string formats: (technically I already enforced validation)
            if (!Byte.TryParse(textDIOByte0.Text, styles, format, out tempByte0) && textDIOByte0.Text != String.Empty)
                bValid = false;
            if (!Byte.TryParse(textDIOByte1.Text, styles, format, out tempByte1) && textDIOByte1.Text != String.Empty)
                bValid = false;
            if (!UInt16.TryParse(textDAC0.Text, styles, format, out tempShort0) && textDAC0.Text != String.Empty)
                bValid = false;
            if (!UInt16.TryParse(textDAC1.Text, styles, format, out tempShort1) && textDAC1.Text != String.Empty)
                bValid = false;

            // If all valid data copy to arrays and write it all:
            if (bValid)
            {
                DIOData[0] = tempByte0;
                DIOData[1] = tempByte1;

                DACData[0] = tempShort0;
                DACData[1] = tempShort1;

                // And finally this is the one function call that is the whole reason for this sample:
                Status = AIOUSB.DACDIO_WriteAll(DeviceIndex, DACData, DIOData);

                // Report call return Status:
                if (Status != 0)
                    textBoxStatus.Text = "Error DACDIO_WriteAll() Status Code: " + Status.ToString();
                else
                    textBoxStatus.Text = "Success DACDIO_WriteAll() Status Code: " + Status.ToString();
            }
            else
            {
                // Invalid characters or blank space:
                textBoxStatus.Text = "Text values must be valid hexidecimal numbers and not blank!";
            }
        }
        
        private String FormatCount(int Count)
        {
            string formatString;
            formatString = String.Format("{0,4:X}", Count);
            return (formatString);
        }

        private void trackBarDAC_Scroll(object sender, EventArgs e)
        {
            //Get button sender Tag (index)
            TrackBar tbarSender = sender as TrackBar;
            UInt32 iTagindex = Convert.ToUInt32(tbarSender.Tag);

            int CounterValue;
            String TempString;

            CounterValue = DACTrack[iTagindex].Value;
            TempString = FormatCount(CounterValue);

            DACText[iTagindex].Text = TempString;
        }
        
        private void textDAC_TextChanged(object sender, EventArgs e)
        {
            // Validate, convert to counts, reset slider:

            bool bValid = true;
            UInt16 tempShort0;
            int CounterValue; 
            NumberStyles styles = System.Globalization.NumberStyles.HexNumber;  // composite number style
            IFormatProvider format = System.Globalization.NumberFormatInfo.CurrentInfo; // can use null as well

            //Get button sender Tag (index)
            TextBox tboxSender = sender as TextBox;
            UInt32 iTagindex = Convert.ToUInt32(tboxSender.Tag);


            if (!UInt16.TryParse(DACText[iTagindex].Text, styles, format, out tempShort0) && DACText[iTagindex].Text != String.Empty)
                bValid = false;

            if (bValid)
            {
                CounterValue = tempShort0;
                DACTrack[iTagindex].Value = CounterValue;
                DACText[iTagindex].BackColor = Color.White;
            }
            else
            {
                // Invalid characters or blank space:
                DACText[iTagindex].BackColor = Color.Red;
                textBoxStatus.Text = "Text values must be valid hexidecimal numbers and not blank!";
            }
        }

        private void checkBox_Click(object sender, EventArgs e)
        {
            bool bHighLow = false;
            UInt32 iTagindex = 0;

            // Get correct checkbox get the checked state

            // Get sender Tag (index) and if check
            CheckBox chkSender = sender as CheckBox;
            if (chkSender != null)
            {
                iTagindex = Convert.ToUInt32(chkSender.Tag);
            }
            bHighLow = chkSender.Checked;


            Int32 BitIndex; 
            Int32 ByteIndex;
            
            //Determine which bit in which byte was clicked.
            BitIndex = (Int32)iTagindex;
            
            if( BitIndex < 8 )
            {
                ByteIndex = 0;
            }
            else 
            {
                ByteIndex = 1;
                BitIndex = BitIndex - 8;
            }

            //Toggle the bit.
            DIOData[ByteIndex] = (Byte)(DIOData[ByteIndex] ^ (1 << BitIndex));

            //Update the edit and graphics.
            DIOText[ByteIndex].Text = DIOData[ByteIndex].ToString("X2");
            
        }

        private void textDIOByte_TextChanged(object sender, EventArgs e)
        {
            // Validate, convert to counts, reset slider:

            bool bValid = true;
            Byte tempByte;
            NumberStyles styles = System.Globalization.NumberStyles.HexNumber;  // composite number style
            IFormatProvider format = System.Globalization.NumberFormatInfo.CurrentInfo; // can use null as well

            //Get button sender Tag (index)
            TextBox tboxSender = sender as TextBox;
            UInt32 iTagindex = Convert.ToUInt32(tboxSender.Tag);


            if (!Byte.TryParse(DIOText[iTagindex].Text, styles, format, out tempByte) && DIOText[iTagindex].Text != String.Empty)
                bValid = false;

            if (bValid)
            {
                DIOData[iTagindex] = tempByte;

                // Test each bit and set check state:
                for (int i = 0; i < numPins; i++)
                {
                    if ((DIOData[i / 8] & (1 << (i % 8))) != 0) // looks complicated but efficient
                        DIOCheckBox[i].Checked = true; // High Level
                    else
                        DIOCheckBox[i].Checked = false; // Low Level
                }
                DIOText[iTagindex].BackColor = Color.White;
            }
            else
            {
                // Invalid characters or blank space:
                DIOText[iTagindex].BackColor = Color.Red;
                textBoxStatus.Text = "Text values must be valid hexidecimal numbers and not blank!";
            }
        }


    }
}
